#if !defined(__LABEL_H)
#define __LABEL_H

class Font;
class Texture;

//Alpha Transparency
typedef DWORD (WINAPI *PSLWA)(HWND, DWORD, BYTE, DWORD);
//typedef DWORD (WINAPI *PULW)(HWND, HDC, POINT, SIZE, HDC, POINT, COLORREF, BLENDFUNCTION, DWORD);

#define WS_EX_LAYERED           0x00080000    

#define LWA_COLORKEY            0x00000001
#define LWA_ALPHA               0x00000002

//#define ULW_ALPHA             0x00000002
//#define AC_SRC_ALPHA			0x01

static PSLWA pSetLayeredWindowAttributes = NULL;
//static PULW pUpdateLayeredWindow = NULL;
static BOOL initialized = FALSE;
//-----------------------------------------------------

class Label
{
public:

	Label(const string &name);
	virtual ~Label();

	void load(HINSTANCE hInstance, HWND box = 0);

	void reconfigure();

	Texture *getBackground() const { return background; }
	Font *getFont() const { return font; }

	bool isAlwaysOnTop() const { return alwaysOnTop; }
	bool isGhosted() const { return ghosted; }
	bool isVisible() const { return visible; }

	int getX() const { return actualx; }
	int getY() const { return actualy; }
	int getHeight() const { return actualheight; }
	int getWidth() const { return actualwidth; }
	int getTransparencyMode() const { return transparencyMode; }

	//int getAlign() { return align; }
	//int getVertAlign() { return vertAlign; }
	//int getUpdateInterval() { return updateInterval; }

	//int getLeftBorder() const { return leftBorder; }
	//int getRightBorder() const { return rightBorder; }
	//int getTopBorder() const { return topBorder; }
	//int getBottomBorder() const { return bottomBorder; }

	int getScrolling() const { return scroll; }

	const HWND &getBox() const { return box; }
	const HWND &gethWnd() const { return hWnd; }
	const string &getName() const { return name; }
	//const string &getText() const { return text; }

	void setBox(HWND newparent);
	void setBackground(Texture *background);
	void setFont(Font *font);
	void setText(const string &text);

	void setAlwaysOnTop(bool alwaysOnTop);
	void setGhosted(bool ghosted);

	void setAlign(int align);
	void setVertAlign(int vertAlign);
	void setUpdateInterval(int updateInterval);

	void setLeftBorder(int leftBorder);
	void setRightBorder(int rightBorder);
	void setTopBorder(int topBorder);
	void setBottomBorder(int bottomBorder);

	void setScrolling(int scrolling);
	void setScrollLimit(int limit);
	
	void setAlpha(int factor = 255);

	void setAnimation(bool start, int loops = -1);

	bool checkRegion(int x, int y, StringList inputlist);
	
	void repaint(bool invalidateCache = false);
	void updateRegion();

	void move(int newx, int newy, int steps = 0, int time = 0, bool moveBy = false);
	void reposition(int newx, int newy, int newwidth, int newheight, int steps = 0, int time = 0);
	void resize(int newwidth, int newheight, int steps = 0, int time = 0, int sizeMode = 0);

	void hide();
	void show();
	void showHide(double timeout);

	void previous();
	void next();
	void update();

	void clipboardCopy(string prefix);
	void clipboardPaste(string prefix);

	void mzscriptvarcopy(string var);
	void posmzscriptvarcopy(string varx = "", string vary = "", bool setmzscript = true);
	void sizemzscriptvarcopy(string varcx = "", string varcy = "", bool setmzscript = true);

	bool bUseFahrenheit;

	string resizeCommand;

	//Scroll AddOns
	bool frozen;
	int scrollBackup;

	//Label/Shortcut Group
	string labelGroup;

	string configSavePath;

	//AutoSize
	int autoWidthMode;
	int autoHeightMode;

	//ModuleHook
	StringList moduleHook;

private:

	HWND hWnd;
	HINSTANCE hInstance;
	HWND box;

	HDC backgroundDC;
	HDC bufferDC;
	HBITMAP backgroundBitmap;
	HBITMAP bufferBitmap;

	int newlineCounter;
	string longestTextLine;

	bool hoverActive;
	bool pressedActive;

	bool useHover;
	bool usePressed;

	bool normalAnim;
	bool hoverAnim;

	bool reallyOverBlend;
	bool paintInProgress;
	int overblendStyle;

	Texture *background;
	Font *font;

	bool smooth;

	bool alwaysOnTop;
	bool visible;
	bool bUsingDefSkin;

	int actualheight;
	int actualwidth;
	int actualx;
	int actualy;

	int scrollPadLength;
	int scrollInterval;
	int scrollSpeed;
	int scroll;
	int scrollPosition;
	UINT scrollLimit;

	//AutoSize
	int autoMinWidth;
	int autoMinHeight;
	int autoMaxWidth;
	int autoMaxHeight;
	int startCenterX;
	int startCenterY;

	int transparencyMode;
	int alphaTransparency;
	bool transparencyNeeded;
	bool alphaEnabled;
	bool alphaFade;
	bool ghosted;

	int align;
	int vertAlign;
	int updateInterval;
	bool alwaysUpdateContent;

	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	string name;

	bool dynamicText;
	vector<string> originalText;
	int current;
	string text;
	string tooltip;

	bool textChange;

	int labelType;

	bool lineBreak;
	int UpdateAlpha;

	bool mousePressed;
	bool mouseInside;

	int focusOnEnter;

	string leftClickCommand;
	string leftDoubleClickCommand;
	string middleClickCommand;
	string middleDoubleClickCommand;
	string rightClickCommand;
	string rightDoubleClickCommand;
	string wheelDownCommand;
	string wheelUpCommand;
	string enterCommand;
	string leaveCommand;
	string dropCommand;
	string textChangeCommand;

	POINT savedpt;
	bool movemodifierkeyPressed;
	int moveButtonPressed;

	bool moveable;
	int moveKey;
	int moveButton;

	StringList labelLeftClickRegions;
	StringList labelRightClickRegions;
	StringList labelMiddleClickRegions;

	//Animations
	int frameCounter;
	int loopCounter;

	int imageLoop;
	int imageFrameDelay;
	int imageFrameCount;
	int hoverimageLoop;
	int hoverimageFrameDelay;
	int hoverimageFrameCount;

protected:

	void autoSize();

	HWND TooltipHints;
	char oldtooltiptext[256];

	void addHint(HWND hWnd, LPSTR caption);
	void updateHint(HWND hWnd, LPSTR caption);
	void removeHint(HWND hWnd);

	BOOL UpdateWindowTransparent(HWND hWnd, unsigned char factor);

	void relayMouseMessageToBox(UINT message, WPARAM wParam, LPARAM lParam);

	virtual void onLButtonDblClk(int x, int y);
	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDblClk(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDblClk(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y);
	virtual void onWheelDown(int x, int y);
	virtual void onWheelUp(int x, int y);
	virtual void onMouseEnter();
	virtual void onMouseLeave();
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);
	virtual void onWindowPosChanged(WINDOWPOS *windowPos);

	virtual bool onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult);

public:

	static LRESULT WINAPI windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
};

#endif
